<?php

namespace agy;

/**
 * WooCommerce Blacklist
 */
class AGY_WooCommerce_Blacklist {

	/**
	 * Constructor for WooCommerce Blacklist
	 */
	public function __construct() {
		add_action( 'woocommerce_product_query', array( $this, 'exclude_products_from_shop' ) );
		add_filter( 'woocommerce_related_products', array( $this, 'exclude_products_from_related' ), 10, 3 );
		add_filter( 'woocommerce_product_get_upsell_ids', array( $this, 'exclude_products_from_upsells' ), 10, 2 );
		add_filter( 'woocommerce_product_get_cross_sell_ids', array( $this, 'exclude_products_from_upsells' ), 10, 2 );
		add_filter( 'woocommerce_products_widget_query_args', array( $this, 'exclude_products_from_widgets' ), 10, 1 );
		add_filter( 'pre_get_posts', array( $this, 'exclude_products_from_search' ) );
		add_action( 'template_redirect', array( $this, 'redirect_excluded_product' ) );
	}

	/**
	 * Get an instance of AGY_WooCommerce_Blacklist
	 *
	 * @return void
	 */
	public static function get_instance() {
		new AGY_WooCommerce_Blacklist();
	}

	/**
	 * Get the current users age
	 *
	 * @return int
	 */
	public static function get_users_age() {

		if ( ! is_user_logged_in() ) {
			return false;
		}

		$user = wp_get_current_user();
		$age  = get_user_meta( $user->ID, 'psag_age_verification', true );
		return $age;
	}

	/**
	 * Check if age verified cookie is set.
	 *
	 * @return boolean
	 */
	public static function is_age_verified() {
		$woocommerce_settings = get_option( 'psag_woocommerce' );

		if ( isset( $woocommerce_settings['psag_woocommerce_deactivate_age_gate'] ) && 'on' === $woocommerce_settings['psag_woocommerce_deactivate_age_gate'] ) {
			return true;
		}

		if ( ! empty( $_COOKIE['agy'] ) ) {
			if ( 'accepted' === $_COOKIE['agy'] ) {
				return true;		
			}
		}
		return false;
	}

	/**
	 * Exclude products from shop archive.
	 *
	 * @param object $q current query object.
	 * @return void
	 */
	public function exclude_products_from_shop( $q ) {

		/* filter the query */
		$ids = array();
		$q   = 'post__not_in';

		/* find products to exclude from shop */
		$products = get_posts( array( 'post_type' => 'product', 'fields' => 'ids', 'posts_per_page' => -1 ) );

		foreach ( $products as $product_id ) {
			$exclude_shop = get_post_meta( $product_id, 'blacklist_shop', true );

			if ( true == $exclude_shop ) {
				$ids[] = $product_id;
			}
		}

		if ( false == self::get_users_age() && false == self::is_age_verified() ) {
			set_query_var( $q, $ids );
		}
	}

	/**
	 * Exclude products from related
	 *
	 * @param array $related_posts array of related posts.
	 * @param int   $product_id current product id.
	 * @param array $args array of arguments.
	 * @return array
	 */
	public function exclude_products_from_related( $related_posts, $product_id, $args ) {

		$ids = array();

		/* find products to exclude from shop */
		$products = get_posts( array( 'post_type' => 'product', 'fields' => 'ids', 'posts_per_page' => -1 ) );

		foreach ( $products as $product_id ) {
			$exclude_shop = get_post_meta( $product_id, 'blacklist_shop', true );

			if ( true == $exclude_shop ) {
				$ids[] = $product_id;
			}
		}

		if ( 0 == self::get_users_age() && ! empty( $ids ) && false == self::is_age_verified() ) {
			return $ids;
		}

		return $related_posts;
	}

	/**
	 * Exclude products from upsells
	 *
	 * @param array  $relatedIds array of related ids.
	 * @param object $product current product object.
	 * @return array
	 */
	public function exclude_products_from_upsells( $relatedIds, $product ) {
		$ids = array();

		/* find products to exclude from shop */
		$products = get_posts( array( 'post_type' => 'product', 'fields' => 'ids', 'posts_per_page' => -1 ) );

		foreach ( $products as $product_id ) {
			$exclude_shop = get_post_meta( $product_id, 'blacklist_shop', true );

			if ( true == $exclude_shop ) {
				$ids[] = $product_id;
			}
		}

		if ( 0 == self::get_users_age() && ! empty( $ids ) && false == self::is_age_verified() ) {
			return $ids;
		}
		return $relatedIds;
	}

	/**
	 * Exclude products from widget
	 *
	 * @param  array $query_args current query arguments.
	 * @return array
	 */
	public function exclude_products_from_widgets( $query_args ) {

		$ids = array();
		$q   = 'post__not_in';

		/* find products to exclude from shop */
		$products = get_posts( array( 'post_type' => 'product', 'fields' => 'ids', 'posts_per_page' => -1 ) );

		foreach ( $products as $product_id ) {
			$exclude_shop = get_post_meta( $product_id, 'blacklist_shop', true );

			if ( true == $exclude_shop ) {
				$ids[] = $product_id;
			}
		}

		if ( 0 == self::get_users_age() && ! empty( $ids ) && false == self::is_age_verified() ) {
			$query_args[ $q ] = $ids;
		}
		return $query_args;
	}
	/**
	 * Exclude products from search
	 *
	 * @param object $query the current query object.
	 * @return void
	 */
	public function exclude_products_from_search( $query ) {

		if ( ! $query->is_admin && $query->is_search ) {

			$ids = array();
			$q   = 'post__not_in';

			/* find products to exclude from shop */
			$products = get_posts( array( 'post_type' => 'product', 'fields' => 'ids', 'posts_per_page' => -1 ) );

			foreach ( $products as $product_id ) {
				$exclude_shop = get_post_meta( $product_id, 'blacklist_shop', true );

				if ( true == $exclude_shop ) {
					$ids[] = $product_id;
				}
			}

			if ( 0 == self::get_users_age() && ! empty( $ids ) && false == self::is_age_verified() ) {
				$query->set( $q, $ids );
			}
		}
		return $query;
	}
	/**
	 * Redirect blacklisted products
	 *
	 * @return void
	 */
	public function redirect_excluded_product() {
		$options = wp_parse_args( get_option( 'psag_options' ), AGY_Admin::get_defaults( 'psag_options' ) );

		if ( function_exists( 'is_product' ) ) {
			if ( is_product() ) {
				$ids = array();

				/* find products to exclude from shop */
				$products = get_posts( array( 'post_type' => 'product', 'fields' => 'ids', 'posts_per_page' => -1 ) );

				foreach ( $products as $product_id ) {
					$exclude_shop = get_post_meta( $product_id, 'blacklist_shop', true );

					if ( true == $exclude_shop ) {
						$ids[] = $product_id;
					}
				}

				if ( 0 == self::get_users_age() && ! empty( $ids ) && false == self::is_age_verified() ) {
					if ( in_array( get_the_id(), $ids ) ) {
						// Maybe add an option here.
						wp_redirect( $options['psag_options_redirect_url'] );
						exit;
					}
				}
			}
		}
	}

	/**
	 * Check if cart has blacklisted products.
	 *
	 * @return boolean
	 */
	public static function has_cart_blacklisted_products() {
		if ( \agy_fs()->is_plan__premium_only( 'pro' ) ) {
			$general_options = wp_parse_args( get_option( 'psag_options' ), AGY_Admin::get_defaults( 'psag_options' ) );
			$whitelist_mode  = $general_options['psag_blacklist_to_whitelist'];

			// set comparable for blacklists.
			$cart_has_blacklist = false;

			foreach ( WC()->cart->get_cart() as $cart_item_key => $cart_item ) {
				$product_id   = $cart_item['product_id'];
				$product      = get_post( $product_id );
				$blacklist    = boolval( get_post_meta( $product_id, 'blacklist', true ) );
				$product_cats = get_the_terms( $product, 'product_cat' );
				$product_tags = get_the_terms( $product, 'product_tag' );

				// Check on category level.
				if ( isset( $product_cats ) && ! empty( $product_cats ) ) {
					foreach ( $product_cats as $product_cat ) {
						$term_blacklist        = get_term_meta( $product_cat->term_id, 'blacklist', true );
						$term_parent_blacklist = get_term_meta( $product_cat->parent, 'blacklist', true );

						if ( true == $term_blacklist || true == $term_parent_blacklist ) {
							if ( isset( $whitelist_mode ) && 'on' === $whitelist_mode ) {
								$cart_has_blacklist = true;
							}
						}
					}
				}

				// Check on tag level.
				if ( isset( $product_tags ) && ! empty( $product_tags ) ) {
					foreach ( $product_tags as $product_tag ) {
						$term_blacklist        = get_term_meta( $product_tag->term_id, 'blacklist', true );
						$term_parent_blacklist = get_term_meta( $product_tag->parent, 'blacklist', true );

						if ( true == $term_blacklist || true == $term_parent_blacklist ) {
							if ( true == $term_blacklist || true == $term_parent_blacklist ) {
								if ( isset( $whitelist_mode ) && 'on' === $whitelist_mode ) {
									$cart_has_blacklist = true;
								}
							}
						}
					}
				}

				// Check on product level.
				if ( isset( $whitelist_mode ) && 'on' === $whitelist_mode && $blacklist ) {
					$cart_has_blacklist = true;
				} elseif( $blacklist ) {
					$cart_has_blacklist = true;
				} else {
					$cart_has_blacklist = false;
				}

			}
			return $cart_has_blacklist;
		}
	}
}
